//#define CGAL_PROFILE

#include <iostream>
#include <vector>
#include <CGAL/Exact_predicates_inexact_constructions_kernel.h>
#include <CGAL/Delaunay_triangulation_2.h>
#include <CGAL/point_generators_2.h>
#include <CGAL/iterator.h>
#include <CGAL/Timer.h>
typedef CGAL::Exact_predicates_inexact_constructions_kernel K;
typedef CGAL::Delaunay_triangulation_2<K>                   DT;
typedef CGAL::Random_points_in_square_2<DT::Point>          Rnd_points_it;
typedef CGAL::Counting_iterator<Rnd_points_it, DT::Point>   Points_iterator;

int main (int argc, char **argv)
{   int N = 1000000; if( argc > 1 )N = atoi(argv[1]); // number of points
    Rnd_points_it   rand_it( 1.0);
    Points_iterator begin(rand_it), end(rand_it,N);
    DT dt;
    CGAL::Timer cost; cost.reset();cost.start();
    std::cout << "  Delaunay triangulation of "<<N<<" points in dim 2"<< std::flush;
    dt.insert(begin,end);
    std::cout << " done in "<<cost.time()/N*1000000<<" micro-seconds per point."<< std::endl;

    std::vector<DT::Vertex_handle> vertices; vertices.reserve(N);
    std::vector<DT::Vertex_handle>::iterator it;
    DT::Finite_vertices_iterator vit;
    for(vit=dt.finite_vertices_begin() ;
	vit != dt.finite_vertices_end(); ++vit)  vertices.push_back(vit);
    std::random_shuffle(vertices.begin(), vertices.end());
    cost.reset();cost.start();
    std::cout << " Removal in dynamic Delaunay triangulation of "
	      <<N<<" points in dim 2"<< std::flush;
    for(it=vertices.begin() ; it!=vertices.end(); ++it) dt.remove(*it);
    std::cout << " done in "<<cost.time()/N*1000000<<" micro-seconds per point."<< std::endl;

    return 0;
}
